; "Chute de neige ou autre brosse..." générateur d'animation (falling-snow-or-other-brush-anim-fr.scm)
; Jean-Marc Chatain (jimcha369@gmail.com)
; 2021/12/11
;
; Crée une copie de l'image en cours puis construit une animation
; avec un effet de chute de neige (ou autre brosse choisie)
; sur toute l'image ou les parties sélectionnées.
; L'animation pourra être enregistrée avec le gif-plug-in.

(define (script-fu-falling-snow-or-other-brush-anim-fr img		; Image à utiliser pour l'animation
								drawable		; Zone de l'image à dessiner, toute l'image ou les parties sélectionnées
								step			; Nombre de calques, taille de brosse et distance entre deux brosses
								time			; Ajuste le délai en millisecondes entre deux calques animés
								layers-number	; Ajuste le nombre de couches de flocons du plus petit au plus grand
								difference		; Différence de taille de flocons entre deux couches de flocons
								first-size		; Première taille de flocons : 1 = le plus petit à 15 = le plus grand
								first-angle		; Régle le premier angle de brosse : -180 à 180
								brush-name		; Définit le nom de la brosse dans le sélecteur de brosse
								brush-color		; Définit la couleur de brosse dans le sélecteur de couleur
								rotation		; Rotation de brosse : -1 = à gauche, 0 = non, 1 = à droite
								wind-force		; -3 vent fort à gauche, 0 pas de vent, 3 vent fort à droite
								blur-level		; Définit le niveau de flou appliqué sur le calque de flocons de neige
								rainy-blur		; Définit le niveau de flou cinétique pour l'effet de pluie sur le calque
								rainy-angle		; Définir l'angle de l'effet de flou de pluie
								opacity			; Ajuste l'opacité du calque de neige de 1 à 100
								mode			; Mode calque de neige vu dans les choix de Gimp
								harden-flakes	; Réglé pour dupliquer les couches de flocons pour les durcir
								reverse-anim	; Réglé pour inverser l'animation pendant la construction
								open-preview	; Définit si la fenêtre d'animation s'ouvre ou non à la fin du script
	)
	(define (rotate-brush brush deviation)		; Change l'angle de la brosse en ajoutant un angle de déviation
	(let* ((sum (+ brush deviation)))
		(if (> sum 180) (- sum 360)				; La valeur de l'angle de brosse est comprise entre -180° et 180°
			(if (< sum -180) (+ sum 360) sum))))

	(define (set-as-integer value)				; Pour transformer une valeur imprécise en valeur entière
	(let* ((sum (+ value 0.5)))
		(inexact->exact (floor sum))))
	
	(let* (	(actual-frame 1)
			(step (set-as-integer step))				; Les sliders donnant des valeurs imprécises, elles doivent être définies comme entières
			(time (set-as-integer time))
			(layers-number (set-as-integer layers-number))
			(difference (set-as-integer difference))
			(first-size (set-as-integer first-size))
			(first-angle (set-as-integer first-angle))
			(rotation (set-as-integer rotation))
			(wind-force (set-as-integer wind-force))
			(blur-level (set-as-integer blur-level))
			(rainy-blur (set-as-integer rainy-blur))
			(rainy-angle (set-as-integer rainy-angle))
			(opacity (set-as-integer opacity))
			(difference (* difference step))			; Définir la différence par incrément de pas
			(points (cons-array 4 'double))				; Tableau pour définir les coordonnées des points
			(image (car (gimp-image-duplicate img)))
			(source-layer (car (gimp-image-get-active-layer image)))
			(width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(bound (cdr (gimp-selection-bounds image)))	; Obtenir la limite de la zone : image ou sélection
			(xmin (car bound))							; Coordonnées du coin supérieur gauche de la zone
			(ymin (cadr bound))
			(xmax (caddr bound))						; Coordonnées du coin inférieur droit de la zone
			(ymax (cadddr bound))
			(zone-width (- xmax xmin))					; Les largeur et hauteur de la zone peuvent être différentes de la taille de l'image
			(zone-height (- ymax ymin))
		)
		(gimp-image-undo-disable image)
		(gimp-context-push)								; Pour enregistrer tous les paramètres de contexte
		(gimp-context-set-brush (car brush-name))
		(gimp-palette-set-foreground brush-color)
		(gimp-context-set-paint-mode LAYER-MODE-NORMAL)
		
		(while (<= actual-frame step)
			(let* (
				(actual-layer 0)
				(actual-size (* first-size step))
				(brush-angle first-angle)
				(base-layer (car (gimp-layer-copy source-layer TRUE)))
				(base-name (string-append "Base "							; Nom de calque de base comme "Base 1 (40ms)(replace)"
										(number->string actual-frame) " ("	; pour créer une animation "prête à enregistrer"
										(number->string time) "ms) (replace)"))
				(snow-layer (car (gimp-layer-new image width height RGBA-IMAGE "Snow" 100 LAYER-MODE-NORMAL)))
				(double-layer 0)
				)
				(gimp-image-insert-layer image base-layer 0 -1)
				(gimp-item-set-name base-layer base-name)
				(gimp-image-insert-layer image snow-layer 0 -1)
		
				(while (< actual-layer layers-number)
					(let* (
						(actual-row 0)
						(mid-size (/ actual-size 2))
						(scrolling (round (/ actual-size step)))
						(nb-row (+ (truncate (/ zone-height actual-size)) 3))
						(nb-brush (+ (truncate (/ zone-width actual-size)) 2 (abs wind-force)))
						(ypos (+ ymin (- (* (+ actual-frame actual-layer -1) scrolling) mid-size actual-size)))
						(brush-deviation 0)
						)
						(if (<> (+ nb-brush wind-force) 0)									; Pour éviter l'erreur "division par zéro"
						(set! brush-deviation (* (/ 360 (+ nb-brush wind-force)) rotation))
						)
						(gimp-context-set-brush-size actual-size)
						
						(while (< actual-row nb-row)
							(let* (
								(actual-pos 0)
								(xpos (+ xmin (* (+ actual-frame actual-layer -1) scrolling wind-force) mid-size))
								)
								(if (> wind-force 0)
								 (set! xpos (- xpos (* actual-size wind-force) actual-size))
								)
								(while (< actual-pos nb-brush)
									(and (> ypos (- ymin mid-size)) (< ypos (+ ymax mid-size))		; Dessiner seulement si la brosse est dans la zone
										(and  (> xpos (- xmin mid-size)) (< xpos (+ xmax mid-size))
											(begin
												(aset points 0 xpos)								; Définir les coordonnées (x0, y0)
												(aset points 1 ypos)
												(aset points 2 xpos)								; Définir les coordonnées (x1, y1)
												(aset points 3 ypos)
												(gimp-context-set-brush-angle brush-angle)
												(gimp-paintbrush-default snow-layer 4 points)
											)
										)
									)
									(set! brush-angle (rotate-brush brush-angle brush-deviation))	; Changer l'angle de la brosse pour un meilleur aspect de neige
									(set! xpos (+ xpos actual-size))
									(set! actual-pos (+ actual-pos 1))
								)
								(set! ypos (+ ypos actual-size))
								(set! actual-row (+ actual-row 1))
							)
						)
						(set! brush-angle (rotate-brush brush-angle brush-deviation))
						(set! actual-size (+ actual-size difference))
						(set! actual-layer (+ actual-layer 1))
					)
				)
				(if (> blur-level 0)
				(plug-in-gauss RUN-NONINTERACTIVE image snow-layer blur-level blur-level 1))
				(if (> rainy-blur 0)
				(plug-in-mblur RUN-NONINTERACTIVE image snow-layer 0 rainy-blur (- 90 rainy-angle) 0 0))
				(if (= harden-flakes TRUE)
					(begin
						(set! double-layer (car (gimp-layer-copy snow-layer TRUE)))		; Durcir la couche de flocons en la dupliquant
						(gimp-image-insert-layer image double-layer 0 -1)
						(set! snow-layer (car (gimp-image-merge-down image double-layer 0)))
					)
				)
				(gimp-layer-set-opacity snow-layer opacity)
				(gimp-layer-set-mode snow-layer mode)
				(set! base-layer (car (gimp-image-merge-down image snow-layer 0)))
				(if (= reverse-anim TRUE)												; Envoyer le calque de base en bas pour inverser l'animation
					(gimp-image-lower-item-to-bottom image base-layer))
				(set! actual-frame (+ actual-frame 1))
			)
		)
		(gimp-image-remove-layer image source-layer)
		(gimp-selection-none image)						; Si la partie dessinable était une sélection, la supprimer
		(gimp-context-pop)								; Rétablir les paramètres de contexte précédents
		(gimp-image-undo-enable image)
		(gimp-display-new image)
		(if (= open-preview TRUE)
			(plug-in-animationplay RUN-NONINTERACTIVE image drawable))
	)
)

(script-fu-register "script-fu-falling-snow-or-other-brush-anim-fr"
	_"_Chute de neige ou autre brosse..."
	_"Crée une image multi-calques avec un effet de chute de neige (ou autre brosse) sur toute l'image ou les parties sélectionnées."
	"Jean-Marc Chatain (jimcha369@gmail.com)"
	"Jean-Marc Chatain"
	"Décembre 2021"
	"RGB* GRAY*"
	SF-IMAGE		"Image" 0
	SF-DRAWABLE		"Drawable" 0
	SF-ADJUSTMENT	_"Nombre de calques\n(Et la taille de base de brosse)"		'(25 5 50 1 1 0 0)
	SF-ADJUSTMENT	_"Délai en ms"					'(50 40 100 10 10 0 0)
	SF-ADJUSTMENT	_"Nombre de couches de flocons"	'(5 1 15 1 1 0 0)
	SF-ADJUSTMENT	_"Différence de taille de flocons\n(Fois la taille de base de brosse)"	'(1 1 2 1 1 0 0)
	SF-ADJUSTMENT	_"Taille de la première brosse\n(Fois la taille de base de brosse)"		'(3 1 15 1 1 0 0)
	SF-ADJUSTMENT	_"Angle de la première brosse"	'(0 -180 180 1 10 0 0)
	SF-BRUSH		_"Nom de la brosse"			'("Bristles 03" 100 3 0)
	SF-COLOR		_"Couleur de la brosse"		'(255 255 255)
	SF-ADJUSTMENT	_"Rotation de la brosse\n-1=à gauche 0=non 1=à droite"	'(-1 -1 1 1 1 0 0)
	SF-ADJUSTMENT	_"Force du vent\n-3-2-1=gauche 0=non 1 2 3=droite"		'(0 -3 3 1 1 0 0)
	SF-ADJUSTMENT	_"Niveau de flou"			'(0 0 5 1 1 0 0)
	SF-ADJUSTMENT	_"Flou de pluie en pixels"	'(0 0 30 1 1 0 0)
	SF-ADJUSTMENT	_"Angle de la pluie en degrés\n-72-64-45 0 45 64 72\npour s'aligner sur le vent"	'(0 -90 90 1 10 0 0)
	SF-ADJUSTMENT	_"Opacité"					'(100 1 100 1 10 0 0)
	SF-OPTION		_"Mode de calque"
	'(_"Normal (ancien)" _"Dissoudre" _"Derrière (ancien)" _"Multiplier (ancien)" _"Écran (ancien)" _"Superposer (ancien)"
	_"Différence (ancien)" _"Addition (ancien)" _"Soustraction (ancien)" _"Assombrir seulement (ancien)" _"Éclaircir seulement (ancien)"
	_"Teinte TSV (ancien)" _"Saturation TSV (ancien)" _"Couleur TSL (ancien)" _"Valeur TSV (ancien)" _"Diviser (ancien)"
	_"Éclaircir (ancien)" _"Assombrir (ancien)" _"Lumière dure (ancien)" _"Lumière douce (ancien)" _"Extraction de grain (ancien)"
	_"Fusion de grain (ancien)" _"Effacer la couleur (ancien)" _"Superposer" _"Teinte LCh" _"Chroma LCh" _"Couleur LCh" _"Luminosité LCh"
	_"Normal" _"Derrière" _"Multiplier" _"Écran" _"Différence" _"Addition" _"Soustraction" _"Assombrir seulement"
	_"Éclaircir seulement" _"Teinte TSV" _"Saturation TSV" _"Couleur TSL" _"Valeur TSV" _"Diviser" _"Éclaircir" _"Assombrir"
	_"Lumière dure" _"Lumière douce" _"Extraction de grain" _"Fusion de grain" _"Lumière vive" _"Lumière ponctuelle" _"Lumière linéaire"
	_"Mélange dur" _"Exclusion" _"Assombrissement linéaire" _"Assombrir en Luma/Luminance seulement" _"Éclaircir en Luma/Luminance seulement" _"Luminance"
	_"Effacer la couleur" _"Effacer" _"Fusionner" _"Découper" _"Traverser" _"Remplacer" _"Anti-effacement")
	SF-TOGGLE _"Durcir les couches de flocons"	FALSE
	SF-TOGGLE _"Inverser l'animation"			FALSE
	SF-TOGGLE _"Ouvrir la fenêtre d'animation à la fin"	TRUE
)

(script-fu-menu-register "script-fu-falling-snow-or-other-brush-anim-fr"
                         "<Image>/Filters/Animation/Animators")
